/* 
 *
 *  Filename : rpi-hub.cpp
 *
 *  This program makes the RPi as a hub listening to all six pipes from the remote 
 *  sensor nodes ( usually Arduino  or RPi ) and will return the packet back to the 
 *  sensor on pipe0 so that the sender can calculate the round trip delays
 *  when the payload matches.
 *  
 *  Refer to RF24/examples/rpi_hub_arduino/ for the corresponding Arduino sketches 
 * to work with this code.
 *  
 *  CE is connected to GPIO25
 *  CSN is connected to GPIO8 
 *
 *  Refer to RPi docs for GPIO numbers
 *
 *  Author : Stanley Seow
 *  e-mail : stanleyseow@gmail.com
 *  date   : 4th Apr 2013
 *
 */
#include <stdio.h>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <string>
#include "../RF24.h"

using namespace std;
using std::ifstream;
using std::ofstream;
using std::string;
using std::getline;
using std::endl;
using std::cout;

// Radio pipe addresses for the 2 nodes to communicate.
// First pipe is for writing, 2nd, 3rd, 4th, 5th & 6th is for reading...
// Pipe0 in bytes is "serv1" for mirf compatibility
const uint64_t pipes[6] = { 0x7365727631LL, 0xF0F0F0F0E1LL, 0xF0F0F0F0E2LL, 0xF0F0F0F0E3LL, 0xF0F0F0F0E4LL, 0xF0F0F0F0E5LL };

// CE and CSN pins On header using GPIO numbering (not pin numbers)
RF24 radio("/dev/spidev0.0",8000000,18);  // Setup for GPIO 18 CSN
int compteur=0;

int Recherche(std::string str,std::string str2) // Recherche dans string par1 dans la chaine par2 quoi 
{
int a;
std::size_t found = str.find(str2);
  if (found!=std::string::npos)
         a=1; // si trouver retoune 1
    else a=2; // si pas trouver retourne2
return (a);
}

void copie(char * fichier, char * dest)
{
   ifstream ifs(fichier);
   ofstream ofs(dest);
   string ligne;
   while (getline(ifs,ligne))
		ofs<< ligne<< endl;
	ifs.close();
	ofs.close();
}

void lecture(char * fichier)
{
	char receivePayload[32];
		
	ifstream ifs(fichier); // ouverture du fichier

 if (ifs) // ce test choue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
		
        // cette boucle s'arrte ds qu'une erreur de lecture survient
        while (getline(ifs,ligne))
        {
		    radio.stopListening();
			strcpy(receivePayload,ligne.c_str());
			radio.openWritingPipe(pipes[4]);
			radio.write(receivePayload,32);
			usleep(300);
			radio.startListening();
			usleep(300);
            }
			
    }

ifs.close();
remove(fichier);
 }

void enregistrement(char *  fichier,char *  dest,string texte)
{
   int val;
   int etat=0;
   //string texte;
   //texte="#ID60#35.32";


    // le constructeur de ifstream permet d'ouvrir un fichier en lecture
    ifstream ifs(fichier);
    ofstream ofs(dest);

    if (ifs) // ce test choue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue

        // cette boucle s'arrte ds qu'une erreur de lecture survient
        while (getline(ifs,ligne))
        {
            // afficher la ligne  l'cran
            string s1 = texte.substr(0, 6);
            val=Recherche(ligne,s1);
            if (val==2)                     // si a=2 c'est que la ligne ne contient pas
               ofs<< ligne<< endl;
                else
                {
                    ofs<< texte<< endl;
                    etat=1;
                }
        }
        if (etat==0)
            ofs<< texte<< endl;
    }
ifs.close();
ofs.close();
}

void setup(void)
{
	
	//
	// Refer to RF24.h or nRF24L01 DS for settings
	radio.begin();
	radio.enableDynamicPayloads();
	radio.setAutoAck(1);
	radio.setRetries(15,15);
	radio.setDataRate(RF24_250KBPS);
	radio.setPALevel(RF24_PA_MAX);
	radio.setChannel(120);
	radio.setCRCLength(RF24_CRC_16);

	// Open 6 pipes for readings ( 5 plus pipe0, also can be used for reading )
	radio.openWritingPipe(pipes[0]);
	radio.openReadingPipe(1,pipes[1]);
	radio.openReadingPipe(2,pipes[2]);
	radio.openReadingPipe(3,pipes[3]);
	radio.openReadingPipe(4,pipes[4]);
	radio.openReadingPipe(5,pipes[5]);

	//
	// Dump the configuration of the rf unit for debugging
	//

	// Start Listening
	radio.startListening();

	//radio.printDetails();//pour affichage
	//printf("\n\rOutput below : \n\r"); //pour affichage
	usleep(1500);
}

void loop(void)
{
	char receivePayload[32]="";
	uint8_t pipe = 0;
	char entree[50];
	char sortie[50];
	char fichier[50];
	int val=0;
	
	
	 while ( radio.available( &pipe ) ) {
		
		uint8_t len = radio.getDynamicPayloadSize();
		radio.read( receivePayload, len );

		// Display it on screen
		//printf(receivePayload); //pour affichage
		//fichier(receivePayload);
		//printf("\n\r"); //pour affichage
		string s1 = receivePayload;
		val=Recherche(s1,"#ID"); // recherche caratere #
		if (val==1)
		{
		strcpy(entree, "/run/shm/hello.txt");
		strcpy(sortie, "/run/shm/out.txt");
		enregistrement(entree,sortie,s1);
		copie(sortie,entree);
		}
			
		receivePayload[len]=0;
		
	pipe++;
	// reset pipe to 0
	if ( pipe > 5 ) pipe = 0;
	}
	usleep(5000);
	//*******
	compteur++;
	if (compteur==1000){
	strcpy(fichier, "/run/shm/cde.txt");
	lecture(fichier);
	compteur=0;
	}
	//radio.startListening();
	//********
	//sleep(1); //laisse 1 second pour execution des autres programmes
}


int main(int argc, char** argv) 
{
	setup();
	while(1)
		loop();
	
	return 0;
}

